"""A service for working with patches."""
from pathlib import Path
from typing import List

import inject

from emm.options import EmmOptions
from emm.services.evg_cli_service import EvgCliService, PatchInfo
from emm.services.evg_service import EvgService
from emm.services.file_service import FileService


class PatchService:
    """A service for working with patches."""

    @inject.autoparams()
    def __init__(
        self,
        file_service: FileService,
        evg_cli_service: EvgCliService,
        evg_service: EvgService,
        emm_options: EmmOptions,
    ) -> None:
        """
        Initialize the service.

        :param file_service: Service for working with files.
        :param evg_cli_service: Service for working with the evergreen CLI.
        :param evg_service: Service for working with evergreen.
        :param emm_options: Optional about the command.
        """
        self.file_service = file_service
        self.evg_cli_service = evg_cli_service
        self.evg_service = evg_service
        self.emm_options = emm_options

    def create_patch(self, extra_args: List[str]) -> PatchInfo:
        """
        Create an evergreen patch and add any modules to it.

        :param extra_args: Extra arguments to pass to the patch command.
        """
        modules_data = self.evg_service.get_module_map(self.emm_options.evg_project)
        base_patch = self.evg_cli_service.create_patch(extra_args)
        for module, module_data in modules_data.items():
            module_location = Path(module_data.prefix) / module
            if self.file_service.path_exists(module_location):
                self.evg_cli_service.add_module_to_patch(
                    base_patch.patch_id, module, module_location, extra_args
                )

        self.evg_cli_service.finalize_patch(base_patch.patch_id)
        return base_patch

    def create_cq_patch(self, extra_args: List[str]) -> PatchInfo:
        """
        Create an evergreen commit-queue patch.

        :param extra_args: Extra arguments to pass to the patch command.
        """
        modules_data = self.evg_service.get_module_map(self.emm_options.evg_project)
        base_patch = self.evg_cli_service.create_cq_patch(extra_args)
        for module, module_data in modules_data.items():
            module_location = Path(module_data.prefix) / module
            if self.file_service.path_exists(module_location):
                self.evg_cli_service.add_module_to_cq_patch(
                    base_patch.patch_id, module, module_location, extra_args
                )

        self.evg_cli_service.finalize_cq_patch(base_patch.patch_id)
        return base_patch



"""A service for working with evergreen data."""
from functools import lru_cache
from pathlib import Path
from typing import Dict

import inject
from evergreen import EvergreenApi, Manifest, Project
from shrub.v3.evg_project import EvgModule

from emm.services.file_service import FileService


class EvgService:
    """A service for working with evergreen data."""

    @inject.autoparams()
    def __init__(self, evg_api: EvergreenApi, file_service: FileService) -> None:
        """Initialize the service."""
        self.evg_api = evg_api
        self.file_service = file_service

    def get_project_config_location(self, project_id: str) -> str:
        """
        Get the path to the evergreen config file for this project.

        :param project_id: ID of Evergreen project being queried.
        :return: Path to project config file.
        """
        return self.get_evg_project(project_id).remote_path

    def get_project_branch(self, project_id: str) -> str:
        """
        Get the git branch a project works from.

        :param project_id: ID of project to lookup.
        :return: Branch name a project works from.
        """
        return self.get_evg_project(project_id).branch_name

    @lru_cache(maxsize=None)
    def get_evg_project(self, project_id: str) -> Project:
        """
        Get the project configuration for the given evergreen project.

        :param project_id: ID of project to lookup.
        :return: Project configuration for specified project.
        """
        project_config_list = self.evg_api.all_projects(
            project_filter_fn=lambda p: p.identifier == project_id
        )
        if len(project_config_list) != 1:
            raise ValueError(f"Could not find unique project configuration for : '{project_id}'.")
        return project_config_list[0]

    def get_module_locations(self, project_id: str) -> Dict[str, str]:
        """
        Get the paths that project modules are stored.

        :param project_id: ID of project to query.
        :return: Dictionary of modules and their paths.
        """
        module_map = self.get_module_map(project_id)
        return {module.name: module.prefix for module in module_map.values()}

    def get_module_map(self, project_id: str) -> Dict[str, EvgModule]:
        """
        Get a dictionary of known modules and data about them.

        :param project_id: Evergreen ID of project being queried.
        :return: Dictionary of module names to module data.
        """
        project_config_location = self.get_project_config_location(project_id)
        project_config = self.file_service.read_yaml_file(Path(project_config_location))
        return {module["name"]: EvgModule(**module) for module in project_config.get("modules", [])}

    def get_manifest(self, project_id: str, commit_hash: str) -> Manifest:
        """
        Get the manifest for the given commit and evergreen project.

        :param project_id: Evergreen project to query.
        :param commit_hash: Evergreen commit to query.
        :return: Evergreen manifest for given commit.
        """
        return self.evg_api.manifest(project_id, commit_hash)



"""Service for working with git."""
from enum import Enum
from pathlib import Path
from typing import Optional

from plumbum import local


class GitAction(str, Enum):
    """Actions to perform on a git repository."""

    CHECKOUT = "checkout"
    REBASE = "rebase"
    MERGE = "merge"


class GitService:
    """A service for interacting with git."""

    def __init__(self) -> None:
        """Initialize the service."""
        self.git = local.cmd.git

    def perform_git_action(
        self,
        git_action: GitAction,
        revision: str,
        branch_name: Optional[str] = None,
        directory: Optional[Path] = None,
    ) -> None:
        """
        Perform the given git action on a repository.

        :param git_action: Git action to perform.
        :param revision: Git revision to perform action against.
        :param branch_name: Branch name to create if running checkout.
        :param directory: Path to root of repo to operate on.
        """
        if git_action == GitAction.CHECKOUT:
            self.checkout(revision, directory, branch_name)
        elif git_action == GitAction.REBASE:
            self.rebase(revision, directory)
        elif git_action == GitAction.MERGE:
            self.merge(revision, directory)
        else:
            raise ValueError(f"Unknown git action: {git_action.value}")

    def clone(self, name: str, remote_repo: str, directory: Path, branch: Optional[str]) -> None:
        """
        Clone the specified repository to the given location.

        :param name: Local name to use for cloned repo.
        :param remote_repo: Remote repo location.
        :param directory: Directory to clone repo into.
        :param branch: Branch to checkout.
        """
        args = ["clone"]
        if branch is not None:
            args += ["--branch", branch]
        args += [remote_repo, name]

        with local.cwd(self._determine_directory(directory)):
            self.git[args]()

    def fetch(self, directory: Optional[Path] = None) -> None:
        """
        Fetch commit from origin.

        :param directory: Path to root of repo to operate on.
        """
        args = ["fetch", "origin"]
        with local.cwd(self._determine_directory(directory)):
            self.git[args]()

    def checkout(
        self, revision: str, directory: Optional[Path] = None, branch_name: Optional[str] = None
    ) -> None:
        """
        Checkout the given revision.

        :param revision: Revision to checkout.
        :param directory: Directory to execute command at.
        :param branch_name: Name of branch for git checkout.
        """
        args = ["checkout"]
        if branch_name is not None:
            args += ["-b", branch_name]
        args.append(revision)
        with local.cwd(self._determine_directory(directory)):
            self.git[args]()

    def rebase(self, revision: str, directory: Optional[Path] = None) -> None:
        """
        Rebase on the given revision.

        :param revision: Revision to rebase on.
        :param directory: Directory to execute command at.
        """
        args = ["rebase", revision]
        with local.cwd(self._determine_directory(directory)):
            self.git[args]()

    def merge(self, revision: str, directory: Optional[Path] = None) -> None:
        """
        Merge the given revision.

        :param revision: Revision to merge.
        :param directory: Directory to execute command at.
        """
        args = ["merge", revision]
        with local.cwd(self._determine_directory(directory)):
            self.git[args]()

    def current_commit(self, directory: Optional[Path] = None) -> str:
        """
        Get the commit hash of the current HEAD of a repository.

        :param directory: Path to repository to query.
        :return: Git hash of HEAD of repository.
        """
        args = ["rev-parse", "HEAD"]
        with local.cwd(self._determine_directory(directory)):
            return self.git[args]().strip()

    def merge_base(self, commit_a: str, commit_b: str, directory: Optional[Path] = None) -> str:
        """
        Find the common ancestor of the given commits.

        :param commit_a: First commit to compare.
        :param commit_b: Second commit to compare.
        :param directory: Path to repository to query.
        :return: Git hash of common ancestor of the 2 commits.
        """
        args = ["merge-base", commit_a, commit_b]
        with local.cwd(self._determine_directory(directory)):
            return self.git[args]().strip()

    def commit_all(self, commit_message: str, directory: Optional[Path] = None) -> None:
        """
        Get the commit hash of the current HEAD of a repository.

        :param commit_message: Commit message for all the changes.
        :param directory: Directory to execute command at.
        """
        args = ["commit", "--all", "--message", commit_message]
        with local.cwd(self._determine_directory(directory)):
            self.git[args]()

    @staticmethod
    def _determine_directory(directory: Optional[Path] = None) -> Path:
        """
        Determine which directory to run git command in.

        :param directory: Directory containing it repository.
        :return: Path to run git commands in.
        """
        if directory is None:
            return Path(local.cwd)
        elif not directory.is_absolute():
            return Path(local.cwd / directory)
        return directory



"""Service for working with the evergreen CLI."""
import re
from pathlib import Path
from typing import List, NamedTuple

import inject
from plumbum import local

from emm.options import EmmOptions

PATCH_ID_RE = re.compile(r"ID\s:\s(?P<patch_id>\S+)$", flags=re.MULTILINE)
BUILD_URL_RE = re.compile(r"Build\s:\s(?P<build_url>\S+)$", flags=re.MULTILINE)


class PatchInfo(NamedTuple):
    """
    Information about a created patch.

    * patch_id: Id of created patch.
    * patch_url: URL to patch in the evergreen UI.
    """

    patch_id: str
    patch_url: str


class EvgCliService:
    """A service for interacting with the Evergreen CLI."""

    @inject.autoparams()
    def __init__(self, emm_options: EmmOptions) -> None:
        """
        Initialize the service.

        :param emm_options: Options about the command.
        """
        self.emm_options = emm_options
        self.evg_cli = local.cmd.evergreen

    def create_patch(self, extra_args: List[str]) -> PatchInfo:
        """
        Create a patch in evergreen.

        :param extra_args: Extra arguments to add to the patch command.
        :return: Details about the created patch.
        """
        args = ["patch", "--project", self.emm_options.evg_project, "--skip_confirm"]
        args.extend(extra_args)
        cmd_output = self.evg_cli[args]()
        patch_match = PATCH_ID_RE.search(cmd_output)
        build_url_match = BUILD_URL_RE.search(cmd_output)
        if not patch_match or not build_url_match:
            print(cmd_output)
            raise ValueError("Could not recognize output from 'evergreen patch'")

        return PatchInfo(
            patch_id=patch_match.group("patch_id"), patch_url=build_url_match.group("build_url")
        )

    def add_module_to_patch(
        self, patch_id: str, module: str, directory: Path, extra_args: List[str]
    ) -> None:
        """
        Add the specified module to the given patch.

        :param patch_id: ID of patch to add modules to.
        :param module: Name of modules to add.
        :param directory: Directory with the module changes.
        :param extra_args: Extra arguments that were passed to the patch command.
        """
        args = ["patch-set-module", "--module", module, "--patch", patch_id, "--skip_confirm"]
        if "-u" in extra_args or "--uncommitted" in extra_args:
            args.append("--uncommitted")
        if "--large" in extra_args:
            args.append("--large")
        if "--preserve-commits" in extra_args:
            args.append("--preserve-commits")
        with local.cwd(directory):
            self.evg_cli[args]()

    def finalize_patch(self, patch_id: str) -> None:
        """
        Mark the given patch as finalized.

        :param patch_id: ID of patch to finalize.
        """
        args = ["finalize-patch", "--id", patch_id]
        self.evg_cli[args]()

    def create_cq_patch(self, extra_args: List[str]) -> PatchInfo:
        """Create a commit-queue patch."""
        args = ["commit-queue", "merge", "--project", self.emm_options.evg_project, "--pause"]
        args.extend(extra_args)
        cmd_output = self.evg_cli[args]()
        patch_match = PATCH_ID_RE.search(cmd_output)
        build_url_match = BUILD_URL_RE.search(cmd_output)
        if not patch_match or not build_url_match:
            print(cmd_output)
            raise ValueError("Could not recognize output from 'evergreen commit-queue'")

        return PatchInfo(
            patch_id=patch_match.group("patch_id"), patch_url=build_url_match.group("build_url")
        )

    def add_module_to_cq_patch(
        self, patch_id: str, module: str, directory: Path, extra_args: List[str]
    ) -> None:
        """
        Add the specified module to the given commit-queue patch.

        :param patch_id: ID of patch to add modules to.
        :param module: Name of modules to add.
        :param directory: Directory with the module changes.
        :param extra_args: Extra args to pass to command.
        """
        args = [
            "commit-queue",
            "set-module",
            "--module",
            module,
            "--id",
            patch_id,
            "--skip_confirm",
        ]
        args.extend(extra_args)
        with local.cwd(directory):
            self.evg_cli[args]()

    def finalize_cq_patch(self, patch_id: str) -> None:
        """
        Mark the given commit-queue patch as finalized.

        :param patch_id: ID of patch to finalize.
        """
        # evergreen commit-queue merge --resume 5d3b120f1e2d1770d9f2104e
        args = ["commit-queue", "merge", "--resume", patch_id]
        self.evg_cli[args]()



"""A service for working with files."""
from pathlib import Path
from typing import Any, Dict

import yaml


class FileService:
    """A service for working with files."""

    @staticmethod
    def read_yaml_file(file_path: Path) -> Dict[str, Any]:
        """
        Read the given yaml file into a dictionary.

        :param file_path: Path to yaml file to read.
        :return: Dictionary of yaml contents.
        """
        with open(file_path) as file_contents:
            return yaml.safe_load(file_contents)

    @staticmethod
    def write_yaml_file(file_path: Path, contents: Dict[str, Any]) -> None:
        """Write the given contents to the specified file."""
        if not file_path.parent.exists():
            file_path.parent.mkdir(parents=True, exist_ok=True)

        with open(file_path, "w") as file_contents:
            yaml.safe_dump(contents, file_contents)

    @staticmethod
    def path_exists(path: Path) -> bool:
        """Determine if the given path exists."""
        return path.exists()

    @staticmethod
    def create_symlink(target: Path, source: Path) -> None:
        """Create a symlink."""
        target.symlink_to(source)

    @staticmethod
    def rm_symlink(target: Path) -> None:
        """Delete the specified symlink."""
        target.unlink()

    @staticmethod
    def mkdirs(target: Path) -> None:
        """Create directories for path if they don't exist."""
        target.mkdir(parents=True)



"""Configuration options for evg-module-manager."""
import os
from dataclasses import dataclass
from pathlib import Path

DEFAULT_MODULES_PATH = ".."
DEFAULT_EVG_CONFIG = os.path.expanduser("~/.evergreen.yml")
DEFAULT_EVG_PROJECT = "mongodb-mongo-master"
DEFAULT_EVG_PROJECT_CONFIG = "etc/evergreen.yml"


@dataclass
class EmmOptions:
    """
    Options for running the script.

    * modules_directory: Directory to clone modules into.
    * evg_config: Path to evergreen API configuration.
    * evg_project: Evergreen project of base repository.
    """

    modules_directory: Path = Path(DEFAULT_MODULES_PATH)
    evg_config: Path = Path(DEFAULT_EVG_CONFIG)
    evg_project: str = DEFAULT_EVG_PROJECT



"""Service for working with evergreen modules."""
from pathlib import Path
from typing import Dict

import inject
from shrub.v3.evg_project import EvgModule

from emm.options import EmmOptions
from emm.services.evg_service import EvgService
from emm.services.file_service import FileService
from emm.services.git_service import GitService


class ModulesService:
    """A service for working with evergreen modules."""

    @inject.autoparams()
    def __init__(
        self,
        emm_options: EmmOptions,
        evg_service: EvgService,
        git_service: GitService,
        file_service: FileService,
    ) -> None:
        """
        Initialize the service.

        :param emm_options: Configuration options for modules.
        :param evg_service: Service for working with evergreen.
        :param git_service: Service for working with git.
        :param file_service: Service for working with files.
        """
        self.emm_options = emm_options
        self.evg_service = evg_service
        self.git_service = git_service
        self.file_service = file_service

    def enable(self, module_name: str, sync_commit: bool = True) -> None:
        """
        Enable the given module.

        :param module_name: Name of module to enable.
        :param sync_commit: If True, checkout the module commit associated with the base repo.
        """
        module_data = self.get_module_data(module_name)
        modules_dir = self.emm_options.modules_directory
        module_location = modules_dir / module_data.get_repository_name()

        target_dir = Path(module_data.prefix)

        if not self.file_service.path_exists(target_dir):
            self.file_service.mkdirs(target_dir)

        target_location = target_dir / module_name
        if self.file_service.path_exists(target_location):
            raise ValueError(f"Module {module_name} already exists at {target_location}")

        if not module_location.exists():
            self.git_service.clone(
                module_data.get_repository_name(), module_data.repo, modules_dir, module_data.branch
            )

        print(f"Create symlink: {target_location} -> {module_location.resolve()}")
        self.file_service.create_symlink(target_location, module_location.resolve())

        if sync_commit:
            self.sync_module(module_name, module_data)

    def disable(self, module_name: str) -> None:
        """Disable to specified module."""
        module_data = self.get_module_data(module_name)
        target_dir = Path(module_data.prefix)

        target_location = target_dir / module_name
        if not self.file_service.path_exists(target_location):
            raise ValueError(f"Module {module_name} does not exists at {target_location}")

        self.file_service.rm_symlink(target_location)

    def get_module_data(self, module_name: str) -> EvgModule:
        """
        Get data about the specified module.

        :param module_name: Module to query.
        :return: Details about the module.
        """
        modules_data = self.evg_service.get_module_map(self.emm_options.evg_project)
        if module_name not in modules_data:
            raise ValueError(f"Could not find module {module_name} in evergreen project config.")

        return modules_data[module_name]

    def get_all_modules(self, enabled: bool) -> Dict[str, EvgModule]:
        """
        Get a dictionary of all known modules.

        :param enabled: If True only return modules enabled locally.
        :return: Dictionary of module names to module information.
        """
        all_modules = self.evg_service.get_module_map(self.emm_options.evg_project)
        pred = self.is_module_enabled if enabled else lambda _name, _: True
        return {k: v for k, v in all_modules.items() if pred(k, v)}

    def is_module_enabled(self, module_name: str, module_data: EvgModule) -> bool:
        """
        Determine if the given module is enabled locally.

        :param module_name: Name of module to check.
        :param module_data: Data about the module being checked.
        :return: True if module is enabled locally.
        """
        module_location = Path(module_data.prefix) / module_name
        return self.file_service.path_exists(module_location)

    def sync_module(self, module_name: str, module_data: EvgModule) -> None:
        """
        Sync the given module to the commit associated with the base repo in evergreen.

        :param module_name: Name of module being synced.
        :param module_data: Data about the module.
        """
        project_branch = self.evg_service.get_project_branch(self.emm_options.evg_project)
        base_revision = self.git_service.merge_base(project_branch, "HEAD")
        manifest = self.evg_service.get_manifest(self.emm_options.evg_project, base_revision)
        manifest_modules = manifest.modules
        if manifest_modules is None:
            raise ValueError("Modules not found in manifest")

        module_manifest = manifest_modules.get(module_name)
        if module_manifest is None:
            raise ValueError(f"Module not found in manifest: {module_name}")

        module_revision = module_manifest.revision
        module_location = Path(module_data.prefix) / module_name

        self.git_service.fetch(module_location)
        self.git_service.checkout(module_revision, directory=module_location)

    def git_commit_modules(self, commit: str) -> None:
        """
        Git commit all changes to all modules.

        :param commit: Commit content for all changes.
        """
        enabled_modules = self.get_all_modules(True)
        self.git_service.commit_all(commit)
        for module in enabled_modules:
            module_data = self.get_module_data(module)
            module_location = Path(module_data.prefix) / module
            self.git_service.commit_all(commit, module_location)



"""Command line entry point to application."""
import logging
import os.path
import sys
from pathlib import Path
from typing import List

import click
import inject
import structlog
from evergreen import EvergreenApi, RetryingEvergreenApi
from structlog.stdlib import LoggerFactory

from emm.options import DEFAULT_EVG_CONFIG, DEFAULT_EVG_PROJECT, DEFAULT_MODULES_PATH, EmmOptions
from emm.services.modules_service import ModulesService
from emm.services.patch_service import PatchService

LOGGER = structlog.get_logger(__name__)

EXTERNAL_LOGGERS = [
    "evergreen",
    "inject",
    "urllib3",
]


class EmmOrchestrator:
    """An orchestrator for evg-module-manager."""

    @inject.autoparams()
    def __init__(self, modules_service: ModulesService, patch_service: PatchService) -> None:
        """Initialize the orchestrator."""
        self.modules_service = modules_service
        self.patch_service = patch_service

    def enable(self, module_name: str, sync_commit: bool) -> None:
        """
        Enable the specified module.

        :param module_name: Name of module to enable.
        :param sync_commit: If True, checkout the commit associated with the base repo.
        """
        self.modules_service.enable(module_name, sync_commit)

    def disable(self, module_name: str) -> None:
        """Disable the specified module."""
        self.modules_service.disable(module_name)

    def submit_patch(self, extra_args: List[str]) -> None:
        """
        Submit a patch with all the enabled modules.

        :param extra_args: Extra arguments to pass to the patch command.
        """
        patch_info = self.patch_service.create_patch(extra_args)
        print(f"Patch Submitted: {patch_info.patch_url}")

    def submit_cq_patch(self, extra_args: List[str]) -> None:
        """
        Submit a patch to the commit-queue with all the enabled modules.

        :param extra_args: Extra arguments to pass to the patch command.
        """
        patch_info = self.patch_service.create_cq_patch(extra_args)
        print(f"Patch Submitted: {patch_info.patch_url}")

    def display_modules(self, enabled: bool, details: bool) -> None:
        """
        Display the available modules.

        :param enabled: Only display enabled modules.
        :param details: Display module details.
        """
        modules = self.modules_service.get_all_modules(enabled)
        for module_name, module_data in modules.items():
            print(f"- {module_name}")
            if details:
                print(f"\tprefix: {module_data.prefix}")
                print(f"\trepo: {module_data.repo}")
                print(f"\tbranch: {module_data.branch}")

    def git_commit_modules(self, commit_message: str) -> None:
        """
        Git commit all changes to all modules.

        :param commit_message: Commit content for all changes.
        """
        self.modules_service.git_commit_modules(commit_message)


def configure_logging(verbose: bool) -> None:
    """
    Configure logging.

    :param verbose: Enable verbose logging.
    """
    structlog.configure(logger_factory=LoggerFactory())
    level = logging.DEBUG if verbose else logging.INFO
    logging.basicConfig(
        format="[%(asctime)s - %(name)s - %(levelname)s] %(message)s",
        level=level,
        stream=sys.stderr,
    )
    for log_name in EXTERNAL_LOGGERS:
        logging.getLogger(log_name).setLevel(logging.WARNING)


@click.group(context_settings=dict(auto_envvar_prefix="EMM", max_content_width=100))
@click.option(
    "--modules-dir",
    default=DEFAULT_MODULES_PATH,
    type=click.Path(),
    help=f"Directory to store module repositories [default='{DEFAULT_MODULES_PATH}']",
)
@click.option(
    "--evg-config-file",
    default=DEFAULT_EVG_CONFIG,
    type=click.Path(exists=True),
    help=f"Path to file with evergreen auth configuration [default='{DEFAULT_EVG_CONFIG}']",
)
@click.option(
    "--evg-project",
    default=DEFAULT_EVG_PROJECT,
    help=f"Name of Evergreen project [default='{DEFAULT_EVG_PROJECT}']",
)
@click.pass_context
def cli(ctx: click.Context, modules_dir: str, evg_config_file: str, evg_project: str) -> None:
    """Evergreen Module Manager is a tool help simplify the local workflows of evergreen modules."""
    ctx.ensure_object(EmmOptions)
    ctx.obj.modules_directory = Path(modules_dir)
    ctx.obj.evg_config = Path(evg_config_file)
    ctx.obj.evg_project = evg_project

    configure_logging(False)

    evg_config_file = os.path.expanduser(evg_config_file)
    evg_api = RetryingEvergreenApi.get_api(config_file=evg_config_file)

    def dependencies(binder: inject.Binder) -> None:
        binder.bind(EvergreenApi, evg_api)
        binder.bind(EmmOptions, ctx.obj)

    inject.configure(dependencies)


@cli.command(context_settings=dict(max_content_width=100))
@click.pass_context
@click.option("-m", "--module", required=True, help="Name of module to enable.")
@click.option(
    "--sync-commit/--no-sync-commit",
    default=True,
    help="When true, checkout the commit associated with the base repo in evergreen.",
)
def enable(ctx: click.Context, module: str, sync_commit: bool) -> None:
    """
    Enable the specified module in the current repo.

    If the module does not exist locally, it will be cloned.
    """
    orchestrator = EmmOrchestrator()
    orchestrator.enable(module, sync_commit)


@cli.command(context_settings=dict(max_content_width=100))
@click.pass_context
@click.option("-m", "--module", required=True, help="Name of module to enable.")
def disable(ctx: click.Context, module: str) -> None:
    """Disable the specified module in the current repo."""
    orchestrator = EmmOrchestrator()
    orchestrator.disable(module)


@cli.command(
    context_settings=dict(max_content_width=100, ignore_unknown_options=True, allow_extra_args=True)
)
@click.pass_context
def patch(ctx: click.Context) -> None:
    """
    Create an Evergreen patch with changes from the base repo and any enabled modules.

    Any options passed to this command was be forwarded to the `evergreen patch` command. However,
    the following options are already included and should not be added:

    * --skip_confirm, --yes, -y
    * --project, -p
    """
    orchestrator = EmmOrchestrator()
    orchestrator.submit_patch(ctx.args)


@cli.command(
    context_settings=dict(max_content_width=100, ignore_unknown_options=True, allow_extra_args=True)
)
@click.pass_context
def commit_queue(ctx: click.Context) -> None:
    """
    Submit changes from the base repository and any enabled modules to the Evergreen commit queue.

    Any enabled modules with changes with be submitted to the commit queue along with changes
    to the base repository.

    Any options passed to this command was be forwarded to the `evergreen patch` command. However,
    the following options are already included and should not be added:

    * --skip_confirm, --yes, -y
    * --project, -p
    """
    orchestrator = EmmOrchestrator()
    orchestrator.submit_cq_patch(ctx.args)


@cli.command(context_settings=dict(max_content_width=100))
@click.option("--enabled", is_flag=True, default=False, help="Only list enabled modules.")
@click.option("--show-details", is_flag=True, default=False, help="Show details about modules.")
@click.pass_context
def list_modules(ctx: click.Context, enabled: bool, show_details: bool) -> None:
    """List the modules available for the current repo."""
    orchestrator = EmmOrchestrator()
    orchestrator.display_modules(enabled, show_details)


@cli.command(context_settings=dict(max_content_width=100))
@click.option("--commit-message", required=True, help="Commit message to apply.")
@click.pass_context
def git_commit(ctx: click.Context, commit_message: str) -> None:
    """
    Create a git commit of changes in each module.

    Any enabled modules with git tracked changes will be committed with the commit message
    along with changes to the base repository.

    The following options are already included in the git commit command:
    * --all, -a
    * --message, -m
    """
    orchestrator = EmmOrchestrator()
    orchestrator.git_commit_modules(commit_message)


if __name__ == "__main__":
    cli(obj=EmmOptions(), auto_envvar_prefix="EMM")



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['emm', 'emm.services']

package_data = \
{'': ['*']}

install_requires = \
['Inject>=4.3.1,<5.0.0',
 'PyYAML>=5.4.1,<6.0.0',
 'click>=8,<9',
 'evergreen.py>=3.2.7,<4.0.0',
 'plumbum>=1.7.0,<2.0.0',
 'pydantic>=1.8.2,<2.0.0',
 'rich>=10.9.0,<11.0.0',
 'shrub.py>=3.0.0,<4.0.0',
 'structlog>=21.1.0,<22.0.0',
 'xdg>=5.1.1,<6.0.0']

entry_points = \
{'console_scripts': ['evg-module-manager = emm.emm_cli:cli']}

setup_kwargs = {
    'name': 'evg-module-manager',
    'version': '0.1.5',
    'description': 'Manage Evergreen modules locally.',
    'long_description': "# Evergreen Module Manager\n\nManage Evergreen modules in your local environment.\n\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/evg-module-manager) [![PyPI](https://img.shields.io/pypi/v/evg-module-manager.svg)](https://pypi.org/project/evg-module-manager/)\n\n## Table of contents\n\n1. [Description](#description)\n2. [Dependencies](#dependencies)\n3. [Installation](#installation)\n4. [Usage](#usage)\n5. [Contributor's Guide](#contributors-guide)\n    - [Setting up a local development environment](#setting-up-a-local-development-environment)\n    - [linting/formatting](#lintingformatting)\n    - [Running tests](#running-tests)\n    - [Automatically running checks on commit](#automatically-running-checks-on-commit)\n    - [Versioning](#versioning)\n    - [Code Review](#code-review)\n    - [Deployment](#deployment)\n6. [Resources](#resources)\n\n## Description\n\nThe evg-module-manager is a tool to help improve the local workflows of working with modules in\nyour evergreen projects. It will help you keep any modules defined in your local project in sync.\nIt supports the following functionality:\n\n* List what modules are defined in the local project.\n* List what modules are currently active in your local repo.\n* Clone a module repository.\n* Enable/disable modules in your local repo.\n* Create an evergreen patch build that includes changes from the local patch build and all enabled\n  modules.\n* Submit a changes to the commit-queue that includes changes from the local patch build and all\n  enabled modules.\n\n## Dependencies\n\n* Python 3.8 or later\n* git\n* evergreen command line tool\n* [Evergreen config file](https://github.com/evergreen-ci/evergreen/wiki/Using-the-Command-Line-Tool#downloading-the-command-line-tool)\n\n## Installation\n\nWe strongly recommend using a tool like [pipx](https://pypa.github.io/pipx/) to install\nthis tool. This will isolate the dependencies and ensure they don't conflict with other tools.\n\n```bash\n$ pipx install evg-module-manager\n```\n\n## Usage\n\nSee the [usage](docs/usage.md) documentation for details about using this tool.\n\n```bash\nevg-module-manager --help\nUsage: evg-module-manager [OPTIONS] COMMAND [ARGS]...\n\n  Evergreen Module Manager is a tool help simplify the local workflows of evergreen modules.\n\nOptions:\n  --modules-dir PATH      Directory to store module repositories [default='..']\n  --evg-config-file PATH  Path to file with evergreen auth configuration\n                          [default='/Users/user/.evergreen.yml']\n  --evg-project TEXT      Name of Evergreen project [default='mongodb-mongo-master']\n  --help                  Show this message and exit.\n\nCommands:\n  commit-queue  Submit changes from the base repository and any enabled modules to the...\n  disable       Disable the specified module in the current repo.\n  enable        Enable the specified module in the current repo.\n  git-commit    Perform git commit of changes in each module.\n  list-modules  List the modules available for the current repo.\n  patch         Create an Evergreen patch with changes from the base repo and any enabled...\n```\n\n## Contributor's Guide\n\n### Setting up a local development environment\n\nThis project uses [poetry](https://python-poetry.org/) for setting up a local environment.\n\n```bash\ngit clone ...\ncd evg-module-manager\npoetry install\n```\n\n### linting/formatting\n\nThis project uses [black](https://black.readthedocs.io/en/stable/) and\n[isort](https://pycqa.github.io/isort/) for formatting.\n\n```bash\npoetry run black src tests\npoetry run isort src tests\n```\n\n### Running tests\n\nThis project uses [pytest](https://docs.pytest.org/en/6.2.x/) for testing.\n\n```bash\npoetry run pytest\n```\n\n### Automatically running checks on commit\n\nThis project has [pre-commit](https://pre-commit.com/) configured. Pre-commit will run\nconfigured checks at git commit time. To enable pre-commit on your local repository run:\n\n```bash\npoetry run pre-commit install\n```\n\n### Versioning\n\nThis project uses [semver](https://semver.org/) for versioning.\n\nPlease include a description what is added for each new version in `CHANGELOG.md`.\n\n### Code Review\n\nPlease open a Github Pull Request for code review.\n\n### Deployment\n\nDeployment to pypi is automatically triggered on merges to main.\n\n## Resources\n\n* [Evergreen REST documentation](https://github.com/evergreen-ci/evergreen/wiki/REST-V2-Usage)\n",
    'author': 'Dev Prod DAG',
    'author_email': 'dev-prod-dag@mongodb.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/evergreen-ci/evg-module-manager',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
